@@ -413,6 +413,10 @@ def smart_fullname_cmp(fullname_variant, **person):
                                                                      " %s" % d['middlename'][0] if d.get('middlename') else '')),
                        lambda **d: " ".join((d.get('lastname') or '', " %s" % d['firstname'][0] if d.get('firstname') else '')),
                        lambda **d: " ".join((d.get('lastname') or '', " %s" % d['middlename'][0] if d.get('middlename') else '')),
+                       lambda **d: " ".join((d.get('lastname') or '', "%s%s" % (d['middlename'][0] if d.get('middlename') else '',
+                                                                                d['firstname'][0] if d.get('firstname') else ''))),
+                       lambda **d: " ".join((d.get('lastname') or '', "%s%s" % (d['firstname'][0] if d.get('firstname') else '',
+                                                                                d['middlename'][0] if d.get('middlename') else ''))),
                       )
     return any(map(lambda fn_factory: smart_cmp(fn_factory(**person), fullname_variant), fullname_factories))
 
@@ -421,12 +425,13 @@ def smart_match_fullname(text, **person):
     text = text.upper()
     for _ in '.,()"\':0123456789!@#$^%&*_~`</>\\': text = text.replace(_," ")
     all_tokens = filter(None, text.split()) 
-    tokens_variants = (all_tokens, filter(lambda _: len(_) > 1, all_tokens))
+    tokens_variants = filter(lambda _: len(_) > 1, set(map(tuple, (all_tokens, filter(lambda _: len(_) > 1, all_tokens)))))
     for tokens in tokens_variants:
-        for j in xrange(0, len(tokens) - 2):
+        for j in xrange(0, len(tokens)):
             pieces = tokens[j:]
-            possible_names = [ " ".join(pieces[:_]) for _ in (2,3)] + [" ".join(pieces[-2:]),] 
-            for _name in possible_names:
-                if smart_fullname_cmp(_name, **person):
-                    return True
+            if pieces:
+                possible_names = set((" ".join(pieces[:_]) for _ in (2,3)))
+                for _name in possible_names:
+                    if smart_fullname_cmp(_name, **person):
+                        return True
     return False
\ No newline at end of file
